﻿using AutoMapper;
using BizService;
using BlankApp1.Events;
using Model.Dto;
using Model.Entities;
using Newtonsoft.Json;
using PLCTool.Common;
using Prism.Commands;
using Prism.Events;
using Prism.Mvvm;
using Prism.Services.Dialogs;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace PLCTool.ViewModels.BasicConfigViewModel
{
    public class AddPlanViewModel : BindableBase, IDialogAware
    {
        private readonly IDialogService _dialog;
        private readonly IEventAggregator _aggregator;
        private readonly IOptionConfigService _optionConfigService;
        private readonly IBasicPlcTestSchemeService _basicPlcTestSchemeService;
        private readonly IBasicPlcTestSchemeDtlService _basicPlcTestSchemeDtlService;
        private readonly IMapper _mapper;

        private List<OptionConfigDto> _optionConfigs;
        public AddPlanViewModel(IDialogService dialog, IEventAggregator aggregator, IOptionConfigService optionConfigService,IBasicPlcTestSchemeService basicPlcTestSchemeService, IBasicPlcTestSchemeDtlService basicPlcTestSchemeDtlService,IMapper mapper)
        {
            _dialog = dialog;
            _aggregator = aggregator;
            _optionConfigService= optionConfigService;
            _basicPlcTestSchemeService= basicPlcTestSchemeService;
            _basicPlcTestSchemeDtlService= basicPlcTestSchemeDtlService;
            _mapper = mapper;
            CloseCommand = new DelegateCommand(Close);
            SureCommand = new DelegateCommand<string>(Sure);
            ResetCommand = new DelegateCommand<string>(ResetMethod);
            AddDetailCommand = new DelegateCommand<string>(AddPLCDetail);
            EditBeforeCommand = new DelegateCommand<Object>(EditBefore);
            DeleteBeforeCommand = new DelegateCommand<Object>(DeleteBefore);
          
            BeforeConList = new ObservableCollection<BasPlcItemConfigDto>();
            InConList = new ObservableCollection<BasPlcItemConfigDto>();
            OutConList = new ObservableCollection<BasPlcItemConfigDto>();
            GetConfigOption();
        }

        /// <summary>
        /// 获取配置
        /// </summary>
        private void GetConfigOption()
        {
            var configList = _optionConfigService.QueryList();
            _optionConfigs = _mapper.Map<List<OptionConfig>, List<OptionConfigDto>>(configList);
            var tests = _optionConfigs.FindAll(x => x.TypeID == 1);
            TestKinds = new List<string>();
            foreach (var test in tests)
            {
                TestKinds.Add(test.ContentOption);
            }

            //测试项类型
            var judges = _optionConfigs.FindAll(x => x.TypeID == 2);
            foreach (var judge in judges)
            {
                BeforeJudgeKinds.Add(judge.ContentOption);
                InJudgeKinds.Add(judge.ContentOption);
                OutJudgeKinds.Add(judge.ContentOption);
            }

            //判定逻辑
            var logics = _optionConfigs.FindAll(x => x.TypeID == 3);
            foreach (var logic in logics)
            {
                JudgeLogicKinds.Add(logic.ContentOption);
                InJudgeLogicKinds.Add(logic.ContentOption);
                OutJudgeLogicKinds.Add(logic.ContentOption);
               
            }

        }
        /// <summary>
        /// 前置条件编辑
        /// </summary>
        /// <param name="obj"></param>
        /// <exception cref="NotImplementedException"></exception>
        private void EditBefore(Object obj)
        {
            
        }
        /// <summary>
        /// 前置条件删除
        /// </summary>
        /// <param name="obj"></param>
        /// <exception cref="NotImplementedException"></exception>
        private void DeleteBefore(Object obj)
        {
            int id = Convert.ToInt32(obj);
            MessageBoxResult boxResult = MessageBox.Show("确认删除此条数据?", "确认", MessageBoxButton.OKCancel, MessageBoxImage.Question);
            if(boxResult == MessageBoxResult.OK)
            {
                var del = BeforeConList.FirstOrDefault(x => x.Id == id);
                if(del != null)
                {
                    BeforeConList.Remove(del);  
                }

            }
        }
        /// <summary>
        /// 新增
        /// </summary>
        /// <param name="str"></param>
        private void AddPLCDetail(string str)
        {
            DialogParameters parm = new DialogParameters();
            parm.Add("Key", "");
            //弹出详情对话框
            _dialog.ShowDialog("AddDetailView", parm, async callback =>
            {
                if (callback.Result == ButtonResult.OK)
                {
                    BasPlcItemConfigDto returnValue = callback.Parameters.GetValue<BasPlcItemConfigDto>("ReturnValue");
                    switch(str)
                    {
                        case "Before":
                            BeforeConList.Add(returnValue); break;
                        case "In":
                            InConList.Add(returnValue); break;
                        case "Out":
                            OutConList.Add(returnValue); break;
                    }
                }

            });
        }




        #region idialog接口实现
        public string Title { set; get; } = "弹出窗体";

        public event Action<IDialogResult> RequestClose;

        public bool CanCloseDialog()
        {
            return true;
        }

        public void OnDialogClosed()
        {

        }

        public void OnDialogOpened(IDialogParameters parameters)
        {

        }

        #endregion
        #region 私有方法
        /// <summary>
        /// 初始化
        /// </summary>
     
        /// <summary>
        /// 关闭按钮
        /// </summary>
        private void Close()
        {
            RequestClose?.Invoke(new DialogResult(ButtonResult.Cancel));
        }
        /// <summary>
        /// 确认
        /// </summary>
        /// <param name="obj"></param>
        private void Sure(string obj)
        {
            var finsSche = _basicPlcTestSchemeService.FindByName(ScheduleName);
            if( finsSche != null )
            {
                MessageBoxResult boxResult = MessageBox.Show("测试方案名称已存在，请填写其他名称", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                return;
            }
            //增加测试方案到数据库
            BasicPlcTestSchemeDto schDto=new BasicPlcTestSchemeDto();
            schDto.SchemeName = ScheduleName;
            schDto.DeviceName = DeviceName;
            schDto.CreateBy = Appsession.UserName;
            schDto.CreateTime= DateTime.Now;    
            schDto.UpdateTime = DateTime.Now;
            schDto.UpdateBy = Appsession.UserName;
            bas_plc_test_scheme schEntity = _mapper.Map<BasicPlcTestSchemeDto, bas_plc_test_scheme>(schDto);
            _basicPlcTestSchemeService.Add(schEntity);
            //查找测试方案id
            var finsScheHave = _basicPlcTestSchemeService.FindByName(ScheduleName);
            long id = finsScheHave.scheme_id;
            AddDetailsToDb(id);

            RequestClose?.Invoke(new DialogResult(ButtonResult.OK));
        }
        /// <summary>
        /// 方案明细添加到数据库
        /// </summary>
        private void AddDetailsToDb(long scheduleId)
        {
            //前置项json转化
            string prefixJsonStr=ModelToJsonToStr(BeforeSelectJudge, SelectLogic, BeforeDetail, BeforeConList);
            //输入项json转化
            string inJsonStr = ModelToJsonToStr(SelectInJudge, InSelectLogic, InDetail, InConList);
            //输出项json转化
            string outJsonStr = ModelToJsonToStr(SelectOutJudge, OutSelectLogic, OutDetail, OutConList);
            //添加达到测试主表
            //增加测试方案
            BasicPlcTestSchemeDtlDto schDtlDto = new BasicPlcTestSchemeDtlDto();
            schDtlDto.SchemeId = scheduleId;
            schDtlDto.ItemName = TestName;
            switch (SelectTest)
            {
                case "前置项":
                    schDtlDto.ItemType = 0;
                    break;
                case "PLC点位测试项":
                    schDtlDto.ItemType = 1;
                    break;
                case "Robot动作测试":
                    schDtlDto.ItemType = 2;
                    break;
            }
            schDtlDto.Precondition = prefixJsonStr;
            schDtlDto.PreconditionDescribe = BeforeDetail;
            schDtlDto.Action = inJsonStr;
            schDtlDto.ActionDescribe = InDetail;
            schDtlDto.JudgementResult = outJsonStr;
            schDtlDto.JudgementResultDescribe = OutDetail;
            schDtlDto.CreateBy = Appsession.UserName;
            schDtlDto.CreateTime = DateTime.Now;
            schDtlDto.UpdateTime = DateTime.Now;
            schDtlDto.UpdateBy = Appsession.UserName;

            bas_plc_test_scheme_dtl schDtlEntity = _mapper.Map<BasicPlcTestSchemeDtlDto, bas_plc_test_scheme_dtl>(schDtlDto);
            _basicPlcTestSchemeDtlService.Add(schDtlEntity);
        }
        /// <summary>
        /// mode转json
        /// </summary>
        /// <param name="itemType"></param>
        /// <param name="itemLogical"></param>
        /// <param name="description"></param>
        /// <param name="conList"></param>
        /// <returns></returns>

        private string ModelToJsonToStr(string itemType,string itemLogical,string description, ObservableCollection<BasPlcItemConfigDto> conList)
        {
            JsonModel prefixJsonModel = new JsonModel();
            prefixJsonModel.ItemType = itemType;
            prefixJsonModel.ItemLogical = itemLogical;
            prefixJsonModel.Description = description;
            prefixJsonModel.DetailInfo = new List<DetailPLC>();
            foreach (var item in conList)
            {
                DetailPLC plcItem = new DetailPLC();
                plcItem.PlcItem = item.PlcItem;
                plcItem.PlcAddress = item.PlcAddress;
                plcItem.PlcValue = item.PlcValue;
                plcItem.Remark = item.Remark;
                prefixJsonModel.DetailInfo.Add(plcItem);

            }
            string prefixJsonStr = JsonConvert.SerializeObject(prefixJsonModel);
            return prefixJsonStr;
        }
        /// <summary>
        /// 重置
        /// </summary>
        /// <param name="obj"></param>
        private void ResetMethod(string obj)
        {
            
        }
        #endregion



        #region 命令绑定
        public DelegateCommand CloseCommand { set; get; }
        public DelegateCommand<string> SureCommand { set; get; }
        public DelegateCommand<string> ResetCommand { set; get; }
        /// <summary>
        /// 新增对话框按钮
        /// </summary>
        public DelegateCommand<string> AddDetailCommand { set; get; }
        /// <summary>
        /// 表格删除
        /// </summary>
        public DelegateCommand<Object> DeleteBeforeCommand { set; get; }
        /// <summary>
        /// 表格编辑按钮
        /// </summary>
        public DelegateCommand<Object> EditBeforeCommand { set; get; }
        #endregion
        #region 变量绑定
        /// <summary>
        /// 测试方案编码
        /// </summary>
        private string scheduleName;
        public string ScheduleName
        {
            get { return scheduleName; }
            set { scheduleName = value; RaisePropertyChanged(); }
        }
        /// <summary>
        /// 设备名称
        /// </summary>
        private string deviceName;
        public string DeviceName
        {
            get { return deviceName; }
            set { deviceName = value; RaisePropertyChanged(); }
        }
        /// <summary>
        /// 测试项名称
        /// </summary>
        private string testName;
        public string TestName
        {
            get { return testName; }
            set { testName = value; RaisePropertyChanged(); }
        }
        /// <summary>
        /// 测试项
        /// </summary>
        private List<string> testKinds=new List<string>();
        public List<string> TestKinds
        {
            get { return testKinds; }
            set { testKinds = value; RaisePropertyChanged(); }
        }
        /// <summary>
        /// 
        /// </summary>
        private string selectTest;
        public string SelectTest
        {
            get { return selectTest; }
            set { selectTest = value; RaisePropertyChanged(); }
        }
        /// <summary>
        /// 判定类型(前置项）
        /// </summary>
        private List<string> beforeJudgeKinds=new List<string>();
        public List<string> BeforeJudgeKinds
        {
            get { return beforeJudgeKinds; }
            set { beforeJudgeKinds = value; RaisePropertyChanged(); }
        }
        private string beforeSelectJudge;
        public string BeforeSelectJudge
        {
            get { return beforeSelectJudge; }
            set { beforeSelectJudge = value; RaisePropertyChanged(); }
        }
        /// <summary>
        /// 逻辑判断
        /// </summary>
        private List<string> judgeLogicKinds = new List<string>();
        public List<string> JudgeLogicKinds
        {
            get { return judgeLogicKinds; }
            set { judgeLogicKinds = value; RaisePropertyChanged(); }
        }
        private string selectLogic;
        public string SelectLogic
        {
            get { return selectLogic; }
            set { selectLogic = value; RaisePropertyChanged(); }
        }
        private string beforeDetail;
        public string BeforeDetail
        {
            get { return beforeDetail; }
            set { beforeDetail = value; RaisePropertyChanged(); }
        }

        private ObservableCollection<BasPlcItemConfigDto> beforeConList;
        public ObservableCollection<BasPlcItemConfigDto> BeforeConList
        {
            get { return beforeConList; }
            set { beforeConList = value; RaisePropertyChanged(); }
        }

        /// <summary>
        /// 判定类型(输入项）
        /// </summary>
        private List<string> inJudgeKinds = new List<string>();
        public List<string> InJudgeKinds
        {
            get { return inJudgeKinds; }
            set { inJudgeKinds = value; RaisePropertyChanged(); }
        }
        private string selectInJudge;
        public string SelectInJudge
        {
            get { return selectInJudge; }
            set { selectInJudge = value; RaisePropertyChanged(); }
        }
        /// <summary>
        /// 逻辑判断
        /// </summary>
        private List<string> inJudgeLogicKinds = new List<string>();
        public List<string> InJudgeLogicKinds
        {
            get { return inJudgeLogicKinds; }
            set { inJudgeLogicKinds = value; RaisePropertyChanged(); }
        }
        private string inSelectLogic;
        public string InSelectLogic
        {
            get { return inSelectLogic; }
            set { inSelectLogic = value; RaisePropertyChanged(); }
        }
        private string inDetail;
        public string InDetail
        {
            get { return inDetail; }
            set { inDetail = value; RaisePropertyChanged(); }
        }

        private ObservableCollection<BasPlcItemConfigDto> inConList;
        public ObservableCollection<BasPlcItemConfigDto> InConList
        {
            get { return inConList; }
            set { inConList = value; RaisePropertyChanged(); }
        }
        /// <summary>
        /// 判定类型（结果项）
        /// </summary>
        private List<string> outJudgeKinds = new List<string>();
        public List<string> OutJudgeKinds
        {
            get { return outJudgeKinds; }
            set { outJudgeKinds = value; RaisePropertyChanged(); }
        }
        private string selectOutJudge;
        public string SelectOutJudge
        {
            get { return selectOutJudge; }
            set { selectOutJudge = value; RaisePropertyChanged(); }
        }
        /// <summary>
        /// 逻辑判断
        /// </summary>
        private List<string> outJudgeLogicKinds = new List<string>();
        public List<string> OutJudgeLogicKinds
        {
            get { return outJudgeLogicKinds; }
            set { outJudgeLogicKinds = value; RaisePropertyChanged(); }
        }
        private string outSelectLogic;
        public string OutSelectLogic
        {
            get { return outSelectLogic; }
            set { outSelectLogic = value; RaisePropertyChanged(); }
        }

        private string outDetail;
        public string OutDetail
        {
            get { return outDetail; }
            set { outDetail = value; RaisePropertyChanged(); }
        }
        private ObservableCollection<BasPlcItemConfigDto> outConList;
        public ObservableCollection<BasPlcItemConfigDto> OutConList
        {
            get { return outConList; }
            set { outConList = value; RaisePropertyChanged(); }
        }
        #endregion

    }
}



